﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace HeuristicLab.Services.Hive.DataAccess {
  using DT = HeuristicLab.Services.Hive.Common.DataTransfer;

  public class HiveDao : IHiveDao {
    private IContextFactory<HiveDataContext> contextFactory;
    private DataAccess.HiveDataContext db { get { return contextFactory.CurrentContext; } }

    public HiveDao(IContextFactory<HiveDataContext> contextFactory) {
      this.contextFactory = contextFactory;
    }

    #region Job Methods
    public DT.Job GetJob(Guid id) {
      return Convert.ToDto(db.Jobs.SingleOrDefault(x => x.JobId == id));
    }

    public IEnumerable<DT.Job> GetJobs(Expression<Func<Job, bool>> predicate) {
      return db.Jobs.Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddJob(DT.Job dto) {
      var entity = Convert.ToEntity(dto);
      db.Jobs.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.JobId;
    }

    public void UpdateJob(DT.Job dto) {
      var entity = db.Jobs.FirstOrDefault(x => x.JobId == dto.Id);
      if (entity == null) db.Jobs.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteJob(Guid id) {
      var entity = db.Jobs.FirstOrDefault(x => x.JobId == id);
      if (entity != null) db.Jobs.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion

    #region JobData Methods

    public DT.JobData GetJobData(Guid id) {
      return Convert.ToDto(db.JobDatas.SingleOrDefault(x => x.JobId == id));
    }

    public IEnumerable<DT.JobData> GetJobDatas(Expression<Func<JobData, bool>> predicate) {
      return db.JobDatas.Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddJobData(DT.JobData dto) {
      var entity = Convert.ToEntity(dto);
      db.JobDatas.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.JobId;
    }

    public void UpdateJobData(DT.JobData dto) {
      var entity = db.JobDatas.FirstOrDefault(x => x.JobId == dto.Id);
      if (entity == null) db.JobDatas.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteJobData(Guid id) {
      var entity = db.JobDatas.FirstOrDefault(x => x.JobId == id); // check if all the byte[] is loaded into memory here. otherwise work around to delete without loading it
      if (entity != null) db.JobDatas.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion

    #region HiveExperiment Methods
    public DT.HiveExperiment GetHiveExperiment(Guid id) {
      return Convert.ToDto(db.HiveExperiments.SingleOrDefault(x => x.HiveExperimentId == id));
    }

    public IEnumerable<DT.HiveExperiment> GetHiveExperiments(Expression<Func<HiveExperiment, bool>> predicate) {
      return db.HiveExperiments.Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddHiveExperiment(DT.HiveExperiment dto) {
      var entity = Convert.ToEntity(dto);
      db.HiveExperiments.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.HiveExperimentId;
    }

    public void UpdateHiveExperiment(DT.HiveExperiment dto) {
      var entity = db.HiveExperiments.FirstOrDefault(x => x.HiveExperimentId == dto.Id);
      if (entity == null) db.HiveExperiments.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteHiveExperiment(Guid id) {
      var entity = db.HiveExperiments.FirstOrDefault(x => x.HiveExperimentId == id);
      if (entity != null) db.HiveExperiments.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion


    #region Slave Methods
    public DT.Slave GetSlave(Guid id) {
      return Convert.ToDto(db.Resources.OfType<Slave>().SingleOrDefault(x => x.ResourceId == id));
    }

    public IEnumerable<DT.Slave> GetSlaves(Expression<Func<Slave, bool>> predicate) {
      return db.Resources.OfType<Slave>().Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddSlave(DT.Slave dto) {
      var entity = Convert.ToEntity(dto);
      db.Resources.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.ResourceId;
    }

    public void UpdateSlave(DT.Slave dto) {
      var entity = db.Resources.OfType<Slave>().FirstOrDefault(x => x.ResourceId == dto.Id);
      if (entity == null) db.Resources.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteSlave(Guid id) {
      var entity = db.Resources.OfType<Slave>().FirstOrDefault(x => x.ResourceId == id);
      if (entity != null) db.Resources.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion

    #region SlaveGroup Methods
    public DT.SlaveGroup GetSlaveGroup(Guid id) {
      return Convert.ToDto(db.Resources.OfType<SlaveGroup>().SingleOrDefault(x => x.ResourceId == id));
    }

    public IEnumerable<DT.SlaveGroup> GetSlaveGroups(Expression<Func<SlaveGroup, bool>> predicate) {
      return db.Resources.OfType<SlaveGroup>().Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddSlaveGroup(DT.SlaveGroup dto) {
      var entity = Convert.ToEntity(dto);
      db.Resources.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.ResourceId;
    }

    public void UpdateSlaveGroup(DT.SlaveGroup dto) {
      var entity = db.Resources.OfType<SlaveGroup>().FirstOrDefault(x => x.ResourceId == dto.Id);
      if (entity == null) db.Resources.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteSlaveGroup(Guid id) {
      var entity = db.Resources.OfType<SlaveGroup>().FirstOrDefault(x => x.ResourceId == id);
      if (entity != null) db.Resources.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion

    #region Resource Methods
    public DT.Resource GetResource(Guid id) {
      return Convert.ToDto(db.Resources.SingleOrDefault(x => x.ResourceId == id));
    }

    public IEnumerable<DT.Resource> GetResources(Expression<Func<Resource, bool>> predicate) {
      return db.Resources.Where(predicate).Select(x => Convert.ToDto(x));
    }

    public Guid AddResource(DT.Resource dto) {
      var entity = Convert.ToEntity(dto);
      db.Resources.InsertOnSubmit(entity);
      db.SubmitChanges();
      return entity.ResourceId;
    }

    public void UpdateResource(DT.Resource dto) {
      var entity = db.Resources.FirstOrDefault(x => x.ResourceId == dto.Id);
      if (entity == null) db.Resources.InsertOnSubmit(Convert.ToEntity(dto));
      else Convert.ToEntity(dto, entity);
      db.SubmitChanges();
    }

    public void DeleteResource(Guid id) {
      var entity = db.Resources.FirstOrDefault(x => x.ResourceId == id);
      if (entity != null) db.Resources.DeleteOnSubmit(entity);
      db.SubmitChanges();
    }
    #endregion

    #region Authorization Methods
    public bool IsUserAuthorizedForJobs(Guid userId, params Guid[] jobIds) {
      var userIds = from job in db.Jobs // this needs to be fast!
                 where jobIds.Contains(job.JobId)
                 select job.UserId;
      return userIds.All(x => x == userId);
    }
    #endregion


  }
}
